; +-----------------+
; | PureHTML Styles |
; +-----------------+

;-
;- Macros

Macro SetIfInteger(Variable, Keyword)
  If (ReadPreferenceString(Keyword, ""))
    Variable = Val(ReadPreferenceString(Keyword, ""))
  EndIf
EndMacro
Macro SetIfString(Variable, Keyword)
  If (ReadPreferenceString(Keyword, ""))
    Variable = ReadPreferenceString(Keyword, "")
  EndIf
EndMacro
Macro SetIfColor(Variable, Keyword)
  If (ReadPreferenceString(Keyword, ""))
    Variable = ParseColor(ReadPreferenceString(Keyword, ""))
  EndIf
EndMacro


;-
;- Procedures




; Parse a color string
Procedure.i ParseColor(Input.s)
  Protected Result.i = -1
  
  If (FindString(Input, "RGB("))
    Input = StringField(Input, 2, "(")
    Input = RemoveString(Input, " ")
    Input = RTrim(Input, ")")
    Result = RGB(Val(StringField(Input, 1, ",")), Val(StringField(Input, 2, ",")), Val(StringField(Input, 3, ",")))
  Else
    Result = Val(Input)
  EndIf
  
  ProcedureReturn (Result)
EndProcedure






; Load default preferences
Procedure.i LoadDefaults()
  Protected i.i

  ; Formatting defaults
  Config\TabLength    =  2
  Config\LineNumbers  = #True
  Config\HideIDE      = #True
  Config\IncludeLinks = #False
  
  ; Font defaults
  Config\BGColor  = $DFFFFF
  Config\LNColor  = $D7FFFF
  Config\FontName = "Courier New"
  Config\FontSize =  11
  
  ; Read style defaults
  For i = 0 To #PHS_Count - 1
    Style(i)\Color = DefColor(i)
    If (i = #PHS_Keyword)
      Style(i)\Bold = #True
    Else
      Style(i)\Bold = #False
    EndIf
    Style(i)\Italic    = #False
    Style(i)\Underline = #False
    Style(i)\Strike    = #False
  Next i
EndProcedure


; Import preferences
Procedure.i LoadPrefs(File.s)
  Protected Result.i = #False
  Protected i.i, KeyBold.i
  
  ; Attempt to open preferences
  If (File And OpenPreferences(File))
    Result = #True
  EndIf
  If (Result)
  
    ; Read formatting settings
    PreferenceGroup("Global")
    SetIfInteger(Config\TabLength,   "TabLength")
    SetIfInteger(Config\LineNumbers, "EnableLineNumbers")
    SetIfInteger(Config\HideIDE, #PH_Title + "_HideIDEOptions")
    SetIfInteger(Config\IncludeLinks, #PH_Title + "_IncludeLinks")
    SetIfInteger(Style(#PHS_Keyword)\Bold, "EnableKeywordBolding")
    
    ; Read PureBasic-format settings
    PreferenceGroup("Editor")
    SetIfColor(Config\BGColor,    "BackgroundColor")
    SetIfColor(Config\LNColor,    "LineNumberBackColor")
    SetIfString(Config\FontName,  "EditorFontName")
    SetIfInteger(Config\FontSize, "EditorFontSize")
    For i = 0 To #PHS_Count - 1
      SetIfColor(Style(i)\Color, StyleKeyword(i) + "Color")
    Next i
    
    ; Read colors in import/export location
    PreferenceGroup("Colors")
    For i = 0 To #PHS_Count - 1
      SetIfColor(Style(i)\Color, StyleKeyword(i) + "Color")
    Next i
    
    ; Read extra settings, if found
    For i = 0 To #PHS_Count - 1
      PreferenceGroup(#PH_Title + "_" + StyleKeyword(i))
      SetIfColor(Style(i)\Color,       "Color")
      SetIfInteger(Style(i)\Bold,      "Bold")
      SetIfInteger(Style(i)\Italic,    "Italic")
      SetIfInteger(Style(i)\Underline, "Underline")
      SetIfInteger(Style(i)\Strike,    "Strikethrough")
    Next i
    
  EndIf
  If (Result)
    ClosePreferences()
  EndIf
  
  ProcedureReturn (Result)
EndProcedure



; Write a preference in RGB(r, g, b) format
Procedure WritePreferenceRGB(Keyword.s, Color.i)
  Protected Output.s
  
  Output = "RGB(" + Str(Red(Color)) + ", " + Str(Green(Color))
  Output + ", " + Str(Blue(Color)) + ")"
  WritePreferenceString(Keyword, Output)
EndProcedure


; Export settings to a PB-importable file
Procedure.i SavePrefs(File.s)
  Protected Result.i = #False
  Protected i.i
  
  If (CreatePreferences(File))
    Result = #True
  
    ; Write header
    PreferenceComment("PureBasic IDE Exported Preferences")
    PreferenceComment("Exported from " + #PH_Title + " version " + VersionString(#PH_Version))
    PreferenceComment("")
    PreferenceGroup("Sections")
    WritePreferenceInteger("IncludeColors", 1)
    PreferenceComment("")
    PreferenceComment("Color settings")
    PreferenceComment("")
    
    ; Write colors to export location
    PreferenceGroup("Colors")
    For i = 0 To #PHS_Count - 1
      WritePreferenceRGB(StyleKeyword(i) + "Color", Style(i)\Color)
      WritePreferenceInteger(StyleKeyword(i) + "Color_Used", 1)
    Next i
    WritePreferenceRGB("BackgroundColor", Config\BGColor)
    WritePreferenceInteger("BackgroundColor_Used", 1)
    WritePreferenceRGB("LineNumberBackColor", Config\LNColor)
    WritePreferenceInteger("LineNumberBackColor_Used", 1)
    WritePreferenceRGB("ProcedureBackColor", Config\BGColor)
    WritePreferenceInteger("ProcedureBackColor_Used", 0)
    PreferenceComment("")
    
    ; Write formatting settings
    PreferenceGroup("Global")
    WritePreferenceInteger("TabLength", Config\TabLength)
    WritePreferenceInteger("EnableLineNumbers", Config\LineNumbers)
    WritePreferenceInteger(#PH_Title + "_HideIDEOptions", Config\HideIDE)
    WritePreferenceInteger(#PH_Title + "_IncludeLinks", Config\IncludeLinks)
    WritePreferenceInteger("EnableKeywordBolding", Style(#PHS_Keyword)\Bold)
    PreferenceComment("")
    
    ; Write remaining settings to prefs location
    PreferenceGroup("Editor")
    WritePreferenceInteger("BackgroundColor", Config\BGColor)
    WritePreferenceInteger("LineNumberBackColor", Config\LNColor)
    WritePreferenceString("EditorFontName", Config\FontName)
    WritePreferenceInteger("EditorFontSize", Config\FontSize)
    
    ClosePreferences()
  EndIf
  
  ProcedureReturn (Result)
EndProcedure


; Attempt to load PB IDE preferences, if known
Procedure LoadIDEPrefs()
  Config\IOPath = GetCurrentDirectory()
  
  LoadDefaults()
  LoadPrefs(GetEnvironmentVariable("PB_TOOL_Preferences"))
  LoadPrefs(#PH_PrefFile)
  UpdateForm()
EndProcedure



; Import settings to file
Procedure.i TryImport()
  Protected Result.i = #False
  Protected File.s
  
  File = OpenFileRequester("Import", Config\IOPath, "Style Preferences (*.prefs)|*.prefs|All Files (*.*)|*.*", 0)
  If (File)
    Config\IOPath = GetPathPart(File)
    If (LoadPrefs(File))
      UpdateForm()
      Info("Successfully imported settings.")
      Result = #True
    Else
      Warn("Failed to import settings:" + #LF$ + File)
    EndIf
  EndIf
  
  ProcedureReturn (Result)
EndProcedure

; Export settings to file
Procedure.i TryExport()
  Protected Result.i = #False
  Protected File.s
  
  File = SaveFileRequester("Export", Config\IOPath, "Style Preferences (*.prefs)|*.prefs|All Files (*.*)|*.*", 0)
  If (File)
    If ((SelectedFilePattern() = 0) And (LCase(GetExtensionPart(File)) <> "prefs"))
      File + ".prefs"
    EndIf
    Config\IOPath = GetPathPart(File)
    If (SavePrefs(File))
      Info("Successfully exported settings.")
      Result = #True
    Else
      Warn("Failed to export settings:" + #LF$ + File)
    EndIf
  EndIf
  
  ProcedureReturn (Result)
EndProcedure


;-
;  EOF
; IDE Options = PureBasic 5.00 (Windows - x86)
; CursorPosition = 172
; FirstLine = 160
; Folding = --
; EnableXP
; EnableUser