; +------------------+
; | PureHTML Windows |
; +------------------+

;-
;- Constants


; Special "Gadget Height" Constant,
; OS-dependent and affects most property fields
#GH = WLMO(22, 20, 22, 20)



; Gadget IDs
Enumeration
  #PHG_IOFrame
    #PHG_InFileOption
      #PHG_InFile
      #PHG_InFileBrowse
      #PHG_InFileOpen
    #PHG_OutFileText
      #PHG_OutFile
      #PHG_OutFileBrowse
      #PHG_OutFileOpen
    #PHG_InPathOption
      #PHG_InPath
      #PHG_InPathBrowse
      #PHG_InPathOpen
  
  #PHG_ConfigFrame
    #PHG_FontText
    #PHG_FontName
    #PHG_FontSize
    #PHG_FontPick
    #PHG_BGText
    #PHG_BGColor
    #PHG_TabText
    #PHG_TabLength
    #PHG_LineNumbers
      #PHG_LNColor
    #PHG_HideIDE
    #PHG_Links
  
  #PHG_StyleFrame
    #PHG_StyleText
    #PHG_StyleBox
    #PHG_StyleColorText
    #PHG_StyleColor
    #PHG_StyleBold
    #PHG_StyleItalic
    #PHG_StyleUnderline
    #PHG_StyleStrike
  
  #PHG_Generate
  #PHG_OpenDone
  #PHG_Close
  #PHG_Import
  #PHG_Export
  #PHG_About
EndEnumeration

; Menu Item IDs
Enumeration
  #PHM_Generate
  #PHM_Close
  #PHM_About
  ;
  #PHM_CopyRGB
  #PHM_PasteRGB
  #PHM_CopyBGR
  #PHM_PasteBGR
  #PHM_Invert
  ;
  #PHM_Nothing
EndEnumeration

; Window IDs
Enumeration
  #PHW_Main
  #PHW_Canvas
EndEnumeration




;- Variables

Global ClickedCanvas.i = #Null




;-
;- Procedures

Declare.s CSSColor(Color.i)

; Color a CanvasGadget and store color in GadgetData
Procedure ColorCanvas(Gadget.i, Color.i)
  If (StartDrawing(CanvasOutput(Gadget)))
    Box(0, 0, OutputWidth(), OutputHeight(), Color)
    StopDrawing()
  EndIf
  SetGadgetData(Gadget, Color)
EndProcedure

Procedure CanvasMenu(ID.i)
  If (IsGadget(ID))
    ClickedCanvas = ID
    SetMenuItemText(#PHW_Canvas, #PHM_Nothing, "Value: " + CSSColor(GetGadgetData(ID)))
    DisplayPopupMenu(#PHW_Canvas, WindowID(#PHW_Main))
  Else
    ClickedCanvas = #Null
  EndIf
EndProcedure

Procedure.i SwapRGB(Color.i)
  ProcedureReturn RGB(Blue(Color), Green(Color), Red(Color))
EndProcedure

Procedure CopyCanvas(RRGGBB.i = #True)
  If (ClickedCanvas)
    If (RRGGBB)
      SetClipboardText("#" + RSet(Hex(SwapRGB(GetGadgetData(ClickedCanvas))), 6, "0"))
    Else
      SetClipboardText("$" + RSet(Hex(GetGadgetData(ClickedCanvas)), 6, "0"))
    EndIf
    ClickedCanvas = #Null
  EndIf
EndProcedure

Procedure PasteCanvas(RRGGBB.i = #True)
  Protected Input.s = GetClipboardText()

  CompilerIf (#True)
    If (Left(Input, 1) = "#")
      RRGGBB = #True
    ElseIf (Left(Input, 1) = "$")
      RRGGBB = #False
    EndIf
  CompilerEndIf

  Input = Trim(Input, "#")
  Input = Trim(Input, "$")

  If (ClickedCanvas And (Len(Input) = 6))
    If (RRGGBB)
      ColorCanvas(ClickedCanvas, SwapRGB(Val("$" + Input)))
    Else
      ColorCanvas(ClickedCanvas, Val("$" + Input))
    EndIf
    If (ClickedCanvas = #PHG_StyleColor)
      Style(GetGadgetState(#PHG_StyleBox))\Color = GetGadgetData(ClickedCanvas)
    EndIf
    ClickedCanvas = #Null
  EndIf
EndProcedure

Procedure InvertCanvas()
  If (ClickedCanvas)
    ColorCanvas(ClickedCanvas, #White - GetGadgetData(ClickedCanvas))
    If (ClickedCanvas = #PHG_StyleColor)
      Style(GetGadgetState(#PHG_StyleBox))\Color = GetGadgetData(ClickedCanvas)
    EndIf
    ClickedCanvas = #Null
  EndIf
EndProcedure



;-
;- Main Window


; Update Main Window gadgets as appropriate
Procedure UpdateMainWindow()
  Protected i.i

  ; Single-file Mode
  If (GetGadgetState(#PHG_InFileOption))
    SetGadgetState(#PHG_InPathOption, #False)
    DisableGadget(#PHG_InFile,  #False)
    DisableGadget(#PHG_OutFile, #False)
    DisableGadget(#PHG_InPath,  #True)

  ; Multi-file (folder) Mode
  Else
    SetGadgetState(#PHG_InPathOption, #True)
    DisableGadget(#PHG_InFile,  #True)
    DisableGadget(#PHG_OutFile, #True)
    DisableGadget(#PHG_InPath,  #False)
  EndIf
  
  ; InFile Open
  If (GetGadgetText(#PHG_InFile))
    DisableGadget(#PHG_InFileOpen, #False)
  Else
    DisableGadget(#PHG_InFileOpen, #True)
  EndIf
  
  ; OutFile Open
  If (GetGadgetText(#PHG_OutFile))
    DisableGadget(#PHG_OutFileOpen, #False)
  Else
    DisableGadget(#PHG_OutFileOpen, #True)
  EndIf
  
  ; InPath Open
  If (GetGadgetText(#PHG_InPath))
    DisableGadget(#PHG_InPathOpen, #False)
  Else
    DisableGadget(#PHG_InPathOpen, #True)
  EndIf
  
EndProcedure


; Update all style gadgets with the current configuration
Procedure UpdateForm()
  Protected i.i

  ; Update Settings Frame
  SetGadgetText(#PHG_FontName, Config\FontName)
  SetGadgetText(#PHG_FontSize, Str(Config\FontSize))
  ColorCanvas(#PHG_BGColor, Config\BGColor)
  SetGadgetText(#PHG_TabLength, Str(Config\TabLength))
  SetGadgetState(#PHG_LineNumbers, Config\LineNumbers)
  If (Config\LineNumbers)
    HideGadget(#PHG_LNColor, #False)
  Else
    HideGadget(#PHG_LNColor, #True)
  EndIf
  ColorCanvas(#PHG_LNColor, Config\LNColor)
  SetGadgetState(#PHG_HideIDE, Config\HideIDE)
  SetGadgetState(#PHG_Links, Config\IncludeLinks)
  
  ; Update Style Frame
  i = GetGadgetState(#PHG_StyleBox)
  ColorCanvas(#PHG_StyleColor, Style(i)\Color)
  SetGadgetState(#PHG_StyleBold, Style(i)\Bold)
  SetGadgetState(#PHG_StyleItalic, Style(i)\Italic)
  SetGadgetState(#PHG_StyleUnderline, Style(i)\Underline)
  SetGadgetState(#PHG_StyleStrike, Style(i)\Strike)
  
EndProcedure





; Create main window, menu, and shortcuts
Procedure.i CreateMainWindow()
  Protected Result.i = #False
  Protected Flags.i, Parent.i
  Protected y.i, i.i
  
  ; Open main window
  Parent =  Val(GetEnvironmentVariable("PB_TOOL_MainWindow"))
  Flags  = #PB_Window_MinimizeGadget
  If (Parent)
    Flags | #PB_Window_WindowCentered
  Else
    Flags | #PB_Window_ScreenCentered
  EndIf
  Flags  | #PB_Window_Invisible
  If (OpenWindow(#PHW_Main, 0, 0, 480, 360, #PH_Title + " " + VersionString(#PH_Version), Flags, Parent))
    
    
    ; Input/Output frame
    Frame3DGadget(#PHG_IOFrame, 10, 10, 460, 105, "Input/Output")
      y = 30
      OptionGadget(#PHG_InFileOption, 20, y, 100, #GH, "Input File:")
        SetGadgetState(#PHG_InFileOption, #True)
      OptionGadget(#PHG_InPathOption, 20, y + 2*#GH + 5, 100, #GH, "Input Folder:")
      
      ; Input file selection
      StringGadget(#PHG_InFile, 120, y, 240, #GH, "")
        ButtonGadget(#PHG_InFileBrowse, 360, y, #GH, #GH, "...")
        ButtonGadget(#PHG_InFileOpen, 370 + #GH, y, 60, #GH, "Open") : y + #GH
      TextGadget(#PHG_OutFileText, 37, y+3, 80, #GH, "Output File:")
        StringGadget(#PHG_OutFile, 120, y, 240, #GH, "")
        ButtonGadget(#PHG_OutFileBrowse, 360, y, #GH, #GH, "...")
        ButtonGadget(#PHG_OutFileOpen, 370 + #GH, y, 60, #GH, "Open") : y + #GH + 5
      StringGadget(#PHG_InPath, 120, y, 240, #GH, "")
        ButtonGadget(#PHG_InPathBrowse, 360, y, #GH, #GH, "...")
        ButtonGadget(#PHG_InPathOpen, 370 + #GH, y, 60, #GH, "Open") : y + #GH
      
      
      ; Settings frame
      Frame3DGadget(#PHG_ConfigFrame, 10, 120, 225, 165, "Settings")
        y = 140
        TextGadget(#PHG_FontText, 20, y+3, 40, #GH-3, "Font:")
          StringGadget(#PHG_FontName, 60, y, 90, #GH, "Courier New")
          StringGadget(#PHG_FontSize, 155, y, 30, #GH, "11", #PB_String_Numeric)
          ButtonGadget(#PHG_FontPick, 190, y, 30, #GH, "...")
          y + #GH
        TextGadget(#PHG_BGText, 40, y+3, 110, #GH-3, "Background Color:")
          CanvasGadget(#PHG_BGColor, 155, y+1, 65, #GH-2, #PB_Canvas_Border)
            ColorCanvas(#PHG_BGColor, Config\BGColor)
          y + #GH
        TextGadget(#PHG_TabText, 40, y+3, 110, #GH-3, "Tab Size (Spaces):")
          StringGadget(#PHG_TabLength, 155, y, 65, #GH, "", #PB_String_Numeric)
          y + #GH
        CheckBoxGadget(#PHG_LineNumbers, 20, y, 135, #GH, "Show Line Numbers")
          SetGadgetState(#PHG_LineNumbers, #True)
          CanvasGadget(#PHG_LNColor, 155, y+1, 65, #GH-2, #PB_Canvas_Border)
            ColorCanvas(#PHG_LNColor, Config\LNColor)
          y + #GH
        CheckBoxGadget(#PHG_HideIDE, 20, y, 205, #GH, "Hide Appended IDE Options") : y + #GH
          SetGadgetState(#PHG_HideIDE, #True)
        CheckBoxGadget(#PHG_Links, 20, y, 205, #GH, "IncludeFile Links") : y + #GH
          SetGadgetState(#PHG_Links, #False)
      
      ; Syntax Styles frame
      Frame3DGadget(#PHG_StyleFrame, 245, 120, 225, 165, "Syntax Styles")
        y = 150
        TextGadget(#PHG_StyleText, 255, y+3, 100, #GH-3, "Modify Style:", #PB_Text_Center)
          ComboBoxGadget(#PHG_StyleBox, 355, y, 100, #GH)
            For i = 0 To #PHS_Count - 1
              AddGadgetItem(#PHG_StyleBox, i, StyleName(i))
            Next i
            SetGadgetState(#PHG_StyleBox, 0)
            y + #GH
        TextGadget(#PHG_StyleColorText, 255, y+3, 100, #GH-3, "Color:", #PB_Text_Center)
          CanvasGadget(#PHG_StyleColor, 355, y+1, 100, #GH-2, #PB_Canvas_Border) : y + #GH + 5
        CheckBoxGadget(#PHG_StyleBold, 295, y, 60, #GH, "Bold") : y + #GH
        CheckBoxGadget(#PHG_StyleItalic, 295, y, 60, #GH, "Italic") : y - #GH
        CheckBoxGadget(#PHG_StyleUnderline, 355, y, 90, #GH, "Underline") : y + #GH
        CheckBoxGadget(#PHG_StyleStrike, 355, y, 90, #GH, "Strikethrough") : y - #GH
    
      ; Bottom gadget row
      ButtonGadget(#PHG_Generate, 20, 295, 100, #GH, "Generate", #PB_Button_Default)
        CheckBoxGadget(#PHG_OpenDone, 20, 300 + #GH, 120, #GH, "Open when done")
          SetGadgetState(#PHG_OpenDone, #True)
      ButtonGadget(#PHG_Import, WindowWidth(#PHW_Main)/2 - 60, 295,       120, #GH, "Import Settings")
      ButtonGadget(#PHG_Export, WindowWidth(#PHW_Main)/2 - 60, 295 + #GH, 120, #GH, "Export Settings")
      ButtonGadget(#PHG_Close, 360, 295, 100, #GH, "Close")
      ButtonGadget(#PHG_About, WindowWidth(#PHW_Main)-#GH, WindowHeight(#PHW_Main)-#GH, #GH, #GH, "?")
    
    ; Keyboard shortcuts
    AddKeyboardShortcut(#PHW_Main, #PB_Shortcut_F5,     #PHM_Generate)
    AddKeyboardShortcut(#PHW_Main, #PB_Shortcut_Escape, #PHM_Close   )
    AddKeyboardShortcut(#PHW_Main, #PB_Shortcut_F1,     #PHM_About   )
    
    ; Create ColorCanvas popup menu
    If (CreatePopupMenu(#PHW_Canvas))
      MenuItem(#PHM_Nothing, "")
      MenuBar()
      MenuItem(#PHM_CopyRGB, "Copy #RRGGBB")
      MenuItem(#PHM_PasteRGB, "Paste #RRGGBB")
      MenuBar()
      MenuItem(#PHM_CopyBGR, "Copy $BBGGRR")
      MenuItem(#PHM_PasteBGR, "Paste $BBGGRR")
      MenuBar()
      MenuItem(#PHM_Invert, "Invert Color")
    EndIf
    
    Result = #True
  EndIf
  
  ProcedureReturn (Result)
EndProcedure

; Determine the default path to browse for input files
Procedure.s DefaultBrowsePath()
  Static Path.s = ""
  
  If (Path = "")
    Path = GetEnvironmentVariable("PB_TOOL_FileList")
    If (Path)
      Path = GetPathPart(StringField(Path, 1, #LF$))
    EndIf
  EndIf
  
  If (Path = "")
    Path = GetCurrentDirectory()
  EndIf
  
  ProcedureReturn (Path)
EndProcedure










;-
Procedure MainWindowEvent(Event.i, Gadget.i, Type.i, Menu.i)
  Protected File.s, i.i, k.i
  
  ;- - Gadget Events
  If (Event = #PB_Event_Gadget)
    Select (Gadget)
      
      ; Update main window
      Case #PHG_InFileOption, #PHG_InPathOption
        UpdateMainWindow()
      Case #PHG_InFile, #PHG_OutFile, #PHG_InPath
        If (Type = #PB_EventType_Change)
          UpdateMainWindow()
        EndIf
      
      ; Input file options
      Case #PHG_InFileBrowse
        File = GetGadgetText(#PHG_InFile)
        If (File = "")
          File = DefaultBrowsePath()
        EndIf
        File = OpenFileRequester("Input File", File, #PH_SourcePattern, 0)
        If (File)
          SetGadgetText(#PHG_InFile, File)
          SetGadgetText(#PHG_OutFile, File + ".htm")
          SetGadgetState(#PHG_InFileOption, #True)
          UpdateMainWindow()
        EndIf
      Case #PHG_InFileOpen
        File = GetGadgetText(#PHG_InFile)
        If (FileSize(File) > 0)
          RunProgram(File)
        EndIf
      
      ; Output file settings
      Case #PHG_OutFileBrowse
        File = GetGadgetText(#PHG_OutFile)
        If ((File = "") And GetGadgetText(#PHG_InFile))
          File = GetGadgetText(#PHG_InFile) + ".htm"
        EndIf
        If (File = "")
          File = DefaultBrowsePath()
        EndIf
        File = SaveFileRequester("Output File", File, #PH_OutputPattern, 0)
        If (File)
          If (SelectedFilePattern() = 0)
            Select (LCase(GetExtensionPart(File)))
              Case "htm", "html"
              Default
                File + ".htm"
            EndSelect
          EndIf
          SetGadgetText(#PHG_OutFile, File)
          SetGadgetState(#PHG_InFileOption, #True)
          UpdateMainWindow()
        EndIf
      Case #PHG_OutFileOpen
        File = GetGadgetText(#PHG_OutFile)
        If (FileSize(File) > 0)
          RunProgram(File)
        EndIf
      
      ; Input folder
      Case #PHG_InPathBrowse
        File = GetGadgetText(#PHG_InPath)
        If (File = "")
          File = GetPathPart(GetGadgetText(#PHG_InFile))
        EndIf
        If (File = "")
          File = DefaultBrowsePath()
        EndIf
        File = PathRequester("Input Folder", File)
        If (File)
          SetGadgetText(#PHG_InPath, File)
          SetGadgetState(#PHG_InFileOption, #False)
          UpdateMainWindow()
        EndIf
      Case #PHG_InPathOpen
        File = GetGadgetText(#PHG_InPath)
        If (FileSize(File) = -2)
          RunProgram(File)
        EndIf
      
      ; General settings
      Case #PHG_FontPick
        If (FontRequester(GetGadgetText(#PHG_FontName), Val(GetGadgetText(#PHG_FontName)), 0))
          SetGadgetText(#PHG_FontName, SelectedFontName())
          SetGadgetText(#PHG_FontSize, Str(SelectedFontSize()))
        EndIf
      Case #PHG_BGColor
        If (Type = #PB_EventType_LeftClick)
          k = ColorRequester(GetGadgetData(#PHG_BGColor))
          If (k >= 0)
            ColorCanvas(#PHG_BGColor, k)
          EndIf
        ElseIf (Type = #PB_EventType_RightClick)
          CanvasMenu(Gadget)
        EndIf
      Case #PHG_LineNumbers
        If (GetGadgetState(#PHG_LineNumbers))
          HideGadget(#PHG_LNColor, #False)
        Else
          HideGadget(#PHG_LNColor, #True)
        EndIf
      Case #PHG_LNColor
        If (Type = #PB_EventType_LeftClick)
          k = ColorRequester(GetGadgetData(#PHG_LNColor))
          If (k >= 0)
            ColorCanvas(#PHG_LNColor, k)
          EndIf
        ElseIf (Type = #PB_EventType_RightClick)
          CanvasMenu(Gadget)
        EndIf
      
      ; Style Frame
      Case #PHG_StyleBox
        If (Type = #PB_EventType_Change)
          UpdateForm()
        EndIf
      Case #PHG_StyleColor
        If (Type = #PB_EventType_LeftClick)
          k = ColorRequester(GetGadgetData(#PHG_StyleColor))
          If (k >= 0)
            ColorCanvas(#PHG_StyleColor, k)
            Style(GetGadgetState(#PHG_StyleBox))\Color = k
          EndIf
        ElseIf (Type = #PB_EventType_RightClick)
          CanvasMenu(Gadget)
        EndIf
      Case #PHG_StyleBold, #PHG_StyleItalic, #PHG_StyleUnderline, #PHG_StyleStrike
        Style(GetGadgetState(#PHG_StyleBox))\Bold      = GetGadgetState(#PHG_StyleBold     )
        Style(GetGadgetState(#PHG_StyleBox))\Italic    = GetGadgetState(#PHG_StyleItalic   )
        Style(GetGadgetState(#PHG_StyleBox))\Underline = GetGadgetState(#PHG_StyleUnderline)
        Style(GetGadgetState(#PHG_StyleBox))\Strike    = GetGadgetState(#PHG_StyleStrike   )
        SetActiveGadget(#PHG_StyleBox)
      
      ; Bottom row functions
      Case #PHG_Generate
        TryHighlight()
      Case #PHG_Import
        TryImport()
      Case #PHG_Export
        TryExport()
      Case #PHG_Close
        ExitFlag = #PHE_Quit
      
      Case #PHG_About
        About()
      
    EndSelect
  
  ;- - Menu Events
  ElseIf (Event = #PB_Event_Menu)
    Select (Menu)
      Case #PHM_Generate
        TryHighlight()
      Case #PHM_Close
        ExitFlag = #PHE_Quit
      Case #PHM_About
        About()
      
      Case #PHM_CopyRGB
        CopyCanvas(#True)
      Case #PHM_CopyBGR
        CopyCanvas(#False)
      Case #PHM_PasteRGB
        PasteCanvas(#True)
      Case #PHM_PasteBGR
        PasteCanvas(#False)
      Case #PHM_Invert
        InvertCanvas()
      Case #PHM_Nothing
        ClickedCanvas = #Null
      
    EndSelect
  
  ;- - Window Events
  ElseIf (Event = #PB_Event_CloseWindow)
    ExitFlag = #PHE_Quit
  EndIf
  
EndProcedure

;-
;  EOF
; IDE Options = PureBasic 5.00 (Windows - x86)
; CursorPosition = 231
; FirstLine = 219
; Folding = --
; EnableXP
; EnableUser